import sys

import autograder.api.admin.updatecourse

def run(arguments):
    result = autograder.api.admin.updatecourse.send(arguments, exit_on_error = True)

    if (result['course-updated']):
        print("Course updated.")
    else:
        print("Course not updated. The request was successful, does the course have a source?")

    return 0

def main():
    return run(_get_parser().parse_args())

def _get_parser():
    return autograder.api.admin.updatecourse._get_parser()

if (__name__ == '__main__'):
    sys.exit(main())



"""
Handle timestamp operations.
All datetimes that may be serialized should be produced by this module and handled as strings.
Likewise, all datetimes that come from the autograder should go through here.
"""

import datetime
import re

PRETTY_TIMESTAMP_FORMAT = '%Y-%m-%d %H:%M'
UNKNOWN_TIMESTAMP = "<Unknown Time (%s)>"
MISSING_TIMESTAMP = "<Missing Time>"

def get(source = None, pretty = False, adjust_tz = True):
    if (source == MISSING_TIMESTAMP):
        return source

    if ((source is not None) and ('Unknown Time' in source)):
        return source

    instance, clean_source = _get_as_datetime(source)
    if (instance is None):
        return UNKNOWN_TIMESTAMP % (clean_source)

    return _to_string(instance, pretty = pretty, adjust_tz = adjust_tz)

def _get_as_datetime(source = None):
    if (source is None):
        return datetime.datetime.now(datetime.timezone.utc), None

    if (isinstance(source, datetime.datetime)):
        return source, None

    if (source == ''):
        return None, ''

    if (not isinstance(source, str)):
        raise ValueError("Unknown type ('%s') for timestamp source." % (type(source)))

    # Parse out some cases that Python <= 3.10 cannot deal with.
    # This will remove fractional seconds.
    source = re.sub(r'Z$', '+00:00', source)
    source = re.sub(r'(\d\d:\d\d)(\.\d+)', r'\1', source)

    try:
        return datetime.datetime.fromisoformat(source), None
    except Exception:
        pass

    return None, source

def _to_string(instance, pretty = False, adjust_tz = True):
    if (pretty):
        if (adjust_tz):
            return instance.astimezone().strftime(PRETTY_TIMESTAMP_FORMAT)
        else:
            return instance.strftime(PRETTY_TIMESTAMP_FORMAT)

    return instance.isoformat()



import base64
import sys

import autograder.api.common
import autograder.api.ta.fetchsubmissions

def run(arguments):
    out_path = arguments.out_path
    if (not out_path.endswith('.zip')):
        out_path += '.zip'

    config_data = autograder.api.common.parse_config(arguments)
    success, result = autograder.api.ta.fetchsubmissions.send(config_data.get("server"),
            config_data)

    if (not success):
        print(result)
        return 1

    if (len(result['submission-ids']) == 0):
        print("Could not find any submissions for this assignment.")
        return 0

    print("Found %d submissions." % (len(result['submission-ids'])))
    print("Writting output to '%s'." % (out_path))

    data = base64.b64decode(result['contents'], validate = True)

    with open(out_path, 'wb') as file:
        file.write(data)

    return 0

def _get_parser():
    parser = autograder.api.common.get_argument_parser(description =
            "Get all user's most recent submission directory for an assignment.")

    parser.add_argument('-o', '--out-path', dest = 'out_path',
        action = 'store', type = str, default = 'submissions.zip',
        help = 'Where to write the submission output to (as a zip file) (default: %(default)s).')

    return parser

def main():
    return run(_get_parser().parse_args())

if (__name__ == '__main__'):
    sys.exit(main())



import unittest

import autograder.util.timestamp

class TestTimestamp(unittest.TestCase):
    def test_timestamp_conversions(self):
        # [(input, normal, pretty), ...]
        test_cases = [
            ('2023-09-28', '2023-09-28T00:00:00', '2023-09-28 00:00'),
            ('2023-09-28T00:00:00', '2023-09-28T00:00:00', '2023-09-28 00:00'),
            ('2023-09-28T04:00:20.683684Z', '2023-09-28T04:00:20+00:00', '2023-09-28 04:00'),
            ('2023-09-28T04:00:20.683684+00:00', '2023-09-28T04:00:20+00:00', '2023-09-28 04:00'),
            ('2023-09-28T13:10:44.432050+00:00', '2023-09-28T13:10:44+00:00', '2023-09-28 13:10'),
            ('2023-09-28T13:10:44.43205+00:00', '2023-09-28T13:10:44+00:00', '2023-09-28 13:10'),

            ('abc', '<Unknown Time (abc)>', '<Unknown Time (abc)>'),
        ]

        for i in range(len(test_cases)):
            (source, expected_normal, expected_pretty) = test_cases[i]

            first_normal_timestamp = autograder.util.timestamp.get(source,
                    pretty = False, adjust_tz = False)
            second_normal_timestamp = autograder.util.timestamp.get(first_normal_timestamp,
                    pretty = False, adjust_tz = False)

            self.assertEqual(expected_normal, first_normal_timestamp,
                    "Case %d: [%s] First Normal Timestamp" % (i, source))

            self.assertEqual(expected_normal, second_normal_timestamp,
                    "Case %d: [%s] Second Normal Timestamp" % (i, source))

            first_pretty_timestamp = autograder.util.timestamp.get(source,
                    pretty = True, adjust_tz = False)
            second_pretty_timestamp = autograder.util.timestamp.get(first_pretty_timestamp,
                    pretty = True, adjust_tz = False)

            self.assertEqual(expected_pretty, first_pretty_timestamp,
                    "Case %d: [%s] First Pretty Timestamp" % (i, source))

            self.assertEqual(expected_pretty, second_pretty_timestamp,
                    "Case %d: [%s] Second Pretty Timestamp" % (i, source))

            # Converting from normal to pretty is possible, since no information is list.
            # In the other direction, timezone information is lost.
            normal_pretty_timestamp = autograder.util.timestamp.get(first_normal_timestamp,
                    pretty = True, adjust_tz = False)
            self.assertEqual(expected_pretty, normal_pretty_timestamp,
                    "Case %d: [%s] Normal Then Pretty Timestamp" % (i, source))
